Package: QRscore
Title: Quantile Rank Score
Version: 1.2.0
Date: 2025-03-28
Authors@R: c(
    person("Fanding", "Zhou", email = "zhoufd@berkeley.edu", role = c("cre","aut"),
    comment = c(ORCID = "0000-0003-1306-740X")),
    person("Alan", "Aw", email = "nalawanij@gmail.com", role = c("aut"), 
    comment = c(ORCID = "0000-0001-9455-7878")), 
    person("Dan", "Erdmann-Pham", email = "erdpham@stanford.edu", role = "aut"),
    person("Jonathan", "Fischer", email = "jfischer1@ufl.edu", role = "aut", 
    comment = c(ORCID = "0000-0002-1165-9930")),
    person("Xurui", "Chen", email = "xuruichen@berkeley.edu", role = "ctb")
    )
Description: In genomics, differential analysis enables the discovery of groups of genes implicating important biological processes such as cell differentiation and aging. Non-parametric tests of differential gene expression usually detect shifts in centrality (such as mean or median), and therefore suffer from diminished power against alternative hypotheses characterized by shifts in spread (such as variance). This package provides a flexible family of non-parametric two-sample tests and K-sample tests, which is based on theoretical work around non-parametric tests, spacing statistics and local asymptotic normality (Erdmann-Pham et al., 2022+ [arXiv:2008.06664v2]; Erdmann-Pham, 2023+ [arXiv:2209.14235v2]).  
License: GPL (>= 3)
Depends: R (>= 4.4.0)
Imports: MASS, pscl, arrangements, hitandrun, assertthat, dplyr,
        BiocParallel
Suggests: devtools, DESeq2, knitr, rmarkdown, testthat, BiocStyle
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-10-30 05:52:29 UTC; biocbuild
VignetteBuilder: BiocStyle
URL: https://github.com/songlab-cal/QRscore
BugReports: https://github.com/songlab-cal/QRscore/issues
biocViews: StatisticalMethod, DifferentialExpression, GeneExpression,
        StructuralGenomics, GeneTarget
git_url: https://git.bioconductor.org/packages/QRscore
git_branch: RELEASE_3_22
git_last_commit: 89dd601
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
Author: Fanding Zhou [cre, aut] (ORCID:
    <https://orcid.org/0000-0003-1306-740X>),
  Alan Aw [aut] (ORCID: <https://orcid.org/0000-0001-9455-7878>),
  Dan Erdmann-Pham [aut],
  Jonathan Fischer [aut] (ORCID: <https://orcid.org/0000-0002-1165-9930>),
  Xurui Chen [ctb]
Maintainer: Fanding Zhou <zhoufd@berkeley.edu>
